
"""Package information"""

__project__ = "python-gnutls"
__summary__ = "Python wrapper for the GnuTLS library"
__webpage__ = "https://github.com/AGProjects/python-gnutls"

__version__ = "3.0.0"

__author__ = "Dan Pascu"
__email__ = "dan@ag-projects.com"

__license__ = "LGPL"
__copyright__ = "Copyright 2006-2016 {}".format(__author__)
